﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2017/08 初版
//
// Box Edge Sample Program
// -----------------------------------------------------------------
namespace BoxEdge
{
    public partial class Form_BOX_EDGE_SAMPLE : Form
    {
        //画像メモリ格納領域
        private FVIL.Data.CFviImage m_image = new FVIL.Data.CFviImage();
        //ビデオ入力クラス
        private FVIL.Video.CFviVideo m_video = null;

        //１次エッジ検出クラス(箱指定)
        FVIL.Edge.CFviBoxEdge m_boxedge = new FVIL.Edge.CFviBoxEdge();
        //GDI図形をマウスで操作するオーバーレイクラス
        private WIL.GDI.FigureOverlay m_FigureOverlay = new WIL.GDI.FigureOverlay();
        //表示用オーバーレイクラス宣言
        private FVIL.GDI.CFviOverlay m_overlay = new FVIL.GDI.CFviOverlay();
        //ボックス内直線表示用オーバーレイクラス宣言
        private FVIL.GDI.CFviOverlay m_overlay_box = new FVIL.GDI.CFviOverlay();
        //GDI矩形構造クラス
        private FVIL.GDI.CFviGdiRectangle m_box = new FVIL.GDI.CFviGdiRectangle();
        //GDI線分構造クラス
        private List<FVIL.GDI.CFviGdiLineSegment> m_lines = new List<FVIL.GDI.CFviGdiLineSegment>();
        //GDI点構造クラス
        private FVIL.GDI.CFviGdiPoint m_point1 = new FVIL.GDI.CFviGdiPoint();
        private FVIL.GDI.CFviGdiPoint m_point2 = new FVIL.GDI.CFviGdiPoint();

        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();

        //iniファイル設定フォーム
        Box_Edge_Sample.Form_BROWSEINI formBroseIni = new Box_Edge_Sample.Form_BROWSEINI();
        //iniファイル名（フルパス）
        public string m_strIniFilePath;
        //ビデオデバイスのオープン状態
        private bool m_flgVideoOpened;

        //時間測定用
        private double msec = 0;

        public Form_BOX_EDGE_SAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form_BOX_EDGE_SAMPLE_Shown(object sender, EventArgs e)
        {
            //使用許諾の確認
            Box_Edge_Sample.dialogLicence licence_dialog = new Box_Edge_Sample.dialogLicence();
            if (licence_dialog.ShowDialog(this) != DialogResult.OK)
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

            //iniファイル選択
            fnSetIniFile();

            // 描画imageのセット
            cFviImageView1.Image = m_image;
            //オーバーレイの追加
            cFviImageView1.Display.Overlays.Add(m_overlay);
            cFviImageView1.Display.Overlays.Add(m_FigureOverlay);
            cFviImageView1.Display.Overlays.Add(m_overlay_box);

            //マウスカーソルイベント追加 
            cFviImageView1.MouseMove += new System.Windows.Forms.MouseEventHandler(OnMouseMove);

            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form_BOX_EDGE_SAMPLE_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_flgVideoOpened = false;

            //計測時間テキストボックス初期化
            textBoxTime.Text = "0";

            //トリガモードコンボボックス初期化
            comboBoxTrigger.Items.Clear();
            comboBoxTrigger.Items.Add("ノーマル");
            comboBoxTrigger.Items.Add("内部トリガ");
            comboBoxTrigger.Items.Add("外部トリガ");
            comboBoxTrigger.SelectedIndex = 0;

            //エッジ極性コンボボックス初期化
            comboBoxEdgeDirection.Items.Clear();
            comboBoxEdgeDirection.Items.Add("暗→明方向");
            comboBoxEdgeDirection.Items.Add("明→暗方向");
            comboBoxEdgeDirection.Items.Add("両方向");
            comboBoxEdgeDirection.SelectedIndex = 1;

            //検出線方向コンボボックス初期化
            comboBoxScanDirection.Items.Clear();
            comboBoxScanDirection.Items.Add("右方向");
            comboBoxScanDirection.Items.Add("左方向");
            comboBoxScanDirection.Items.Add("下方向");
            comboBoxScanDirection.Items.Add("上方向");
            comboBoxScanDirection.SelectedIndex = 0;

            // マウスイベント登録
            m_FigureOverlay.AddMouseEventHandler(cFviImageView1);

            //GDI矩形の初期化
            m_box.Pen.Color = Color.Blue;
            //左上、右下、角度
            m_box.St = new FVIL.Data.CFviPoint(0, 0);
            m_box.Ed = new FVIL.Data.CFviPoint(100, 100);
            m_box.Angle = new FVIL.Data.CFviAngle(0.00);
            m_box.Axis = new FVIL.Data.CFviPoint(0, 0);
            //処理矩形表示
            m_box.Enable = true;

            //検出線方向の初期化
            m_point1.Pen.Color = Color.Yellow;
            m_point1.Style = FVIL.GDI.FigureStyle.Arrow;
            m_point1.Size = new System.Drawing.Size(10, 10);
            m_point2.Pen.Color = Color.Yellow;
            m_point2.Style = FVIL.GDI.FigureStyle.Arrow;
            m_point2.Size = new System.Drawing.Size(10, 10);
            
            //GDI追加・有効
            m_FigureOverlay.Figures.Add(m_box);
            m_FigureOverlay.Active = true;
            m_overlay_box.Active = true;

            //検出線表示
            fnLinesDisplay();
        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            //iniファイルの選択
            m_strIniFilePath = "";
            if (formBroseIni.ShowDialog(this) == DialogResult.OK)
            {
                m_strIniFilePath = formBroseIni.strIniFilePath;
            }

            if (m_strIniFilePath.Length > 0)
            {
                try
                {
                    //設定ファイルのロードとデバイスオープン
                    m_video = FVIL.Video.Function.CreateInstance(m_strIniFilePath);
                    //画像サイズ調整
                    m_video.ChangeImageSize(m_image);
                    m_flgVideoOpened = true;

                    //ステータス表示の更新
                    //トリガモードの取得
                    comboBoxTrigger.SelectedIndex = (int)m_video.Trigger;

                    //カメラ露光時間の取得
                    if (m_video.DeviceType != FVIL.Video.DeviceType.RICE001)
                    {
                        numericUpDownExposure.Value = m_video.ExposureTime;
                        comboBoxTrigger.Enabled = true;
                        numericUpDownExposure.Enabled = true;
                    }
                    else
                    {
                        comboBoxTrigger.Enabled = false;
                        numericUpDownExposure.Enabled = false;
                    }

                    buttonGrabOne.Enabled = true;
                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else if (formBroseIni.UseFileFlg == true)
            {
                buttonGrabOne.Enabled = false;
            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void buttonGrabOne_Click(object sender, EventArgs e)
        {
            try
            {
                //描画クリア
                m_overlay.Figures.Clear();
                listViewResult.Items.Clear();

                formBroseIni.UseFileFlg = false;

                m_video.ChangeImageSize(m_image);

                //デバイスタイプ
                FVIL.Video.CFviContinuousGrab contgrab;
                switch (m_video.DeviceType)
                {

                    case FVIL.Video.DeviceType.FHC3312:
                        contgrab = ((FVIL.Video.CFviVideoFHC3312)m_video).ContinuousGrab;
                        ((FVIL.Video.CFviContinuousGrabFHC3312)contgrab).MemoryLock(m_image);
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        contgrab.GrabSync(1);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                    case FVIL.Video.DeviceType.FHC3313:
                        contgrab = ((FVIL.Video.CFviVideoFHC3313)m_video).ContinuousGrab;
                        ((FVIL.Video.CFviContinuousGrabFHC3313)contgrab).MemoryLock(m_image);
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        contgrab.GrabSync(1);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                    default:
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        m_video.GrabImageSync(m_image);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                }
                //測定時間表示
                textBoxTime.Text = msec.ToString("0.000");
                //処理画像設定
                m_boxedge.SrcImages[0] = m_image;

                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void comboBoxTrigger_SelectionChangeCommitted(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.Trigger = (uint)comboBoxTrigger.SelectedIndex;
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void numericUpDownExposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.ExposureTime = (uint)numericUpDownExposure.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// LoadFile ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 入力画像ファイルのロード
        /// </remarks>
        private void buttonLoadImageFile_Click(object sender, EventArgs e)
        {
            //描画クリア
            m_overlay.Figures.Clear();
            listViewResult.Items.Clear();

            formBroseIni.UseFileFlg = true;
            //入力画像ファイルのロード
            openFileDialog1.FileName = null;
            openFileDialog1.Filter = "Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            openFileDialog1.Filter += "|Png ファイル (*.png)|*.png";
            openFileDialog1.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            openFileDialog1.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            openFileDialog1.DefaultExt = "*.bmp";
            if ((openFileDialog1.FileNames.Length <= 0) && (openFileDialog1.ShowDialog(this) == DialogResult.Cancel))
            {
                return;
            }

            try
            {
                FVIL.File.Function.LoadImageFile(openFileDialog1.FileName, m_image, FVIL.PixelMode.Unpacking);
                //処理画像設定
                m_boxedge.SrcImages[0] = m_image;

                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// １次エッジ検出実行ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// １次エッジ検出処理・結果表示
        /// </remarks>
        private void buttonDetection_Click(object sender, EventArgs e)
        {
            //表示初期化
            m_overlay.Figures.Clear();
            listViewResult.Items.Clear();

            //再描画
            cFviImageView1.Refresh();

            //パラメータ設定
            fnBoxEdgePraramSet();
            //１次エッジ検出実行
            if (fnBoxEdgeExec())
            {
                //結果表示
                fnResultDisplay();
            }
        }

        /// <summary>
        /// パラメータ設定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パラメータ設定関数
        /// </remarks>
        private void fnBoxEdgePraramSet()
        {
            //パラメータ設定
            m_boxedge.Box = m_box.ToCFviRectangle();
            m_boxedge.Around = (int)numericUpDownAround.Value;
            switch (comboBoxEdgeDirection.SelectedIndex)
            {
                case 0:
                    m_boxedge.EdgeDirection = FVIL.Edge.Direction.DarkToBright;
                    break;
                case 1:
                    m_boxedge.EdgeDirection = FVIL.Edge.Direction.BrightToDark;
                    break;
                case 2:
                    m_boxedge.EdgeDirection = FVIL.Edge.Direction.Both;
                    break;
            }
            m_boxedge.Foot = (int)numericUpDownFoot.Value;
            m_boxedge.Len = (int)numericUpDownLen.Value;
            m_boxedge.LineWidth = (int)numericUpDownLineWidth.Value;
            m_boxedge.MaxThreshold = (int)numericUpDownMaxThreshold.Value;
            m_boxedge.Number = (int)numericUpDownNumber.Value;
            switch (comboBoxScanDirection.SelectedIndex)
            {
                case 0:
                    m_boxedge.ScanDirection = FVIL.ScanDirection.Right;
                    break;
                case 1:
                    m_boxedge.ScanDirection = FVIL.ScanDirection.Left;
                    break;
                case 2:
                    m_boxedge.ScanDirection = FVIL.ScanDirection.Down;
                    break;
                case 3:
                    m_boxedge.ScanDirection = FVIL.ScanDirection.Up;
                    break;
            }
            m_boxedge.Threshold = (int)numericUpDownThreshold.Value;
        }

        /// <summary>
        /// １次エッジ(箱指定)検出実行
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// １次エッジ(箱指定)検出処理実行関数
        /// </remarks>
        private bool fnBoxEdgeExec()
        {
            bool Result = false;
            try
            {
                //時間測定開始
                m_timer.Start();
                //１次エッジ(箱指定)検出処理実行
                m_boxedge.Execute();
                //時間測定終了
                msec = m_timer.Stop();
                //測定時間表示
                textBoxTime.Text = msec.ToString("0.000");
                Result = true;
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("エラーが発生しました。ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return (Result);
        }

        /// <summary>
        /// 結果表示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 結果表示関数
        /// </remarks>
        private void fnResultDisplay()
        {
            //描画: 検出位置
            for (int i = 0; i < m_boxedge.Result.Count; i++)
            {
                FVIL.Edge.CFviEdgeData edge = m_boxedge.Result[i];

                //＋マーク
                FVIL.GDI.CFviGdiPoint ol_position = new FVIL.GDI.CFviGdiPoint(edge.Position);
                ol_position.Size = new Size(5, 5);
                ol_position.Style = FVIL.GDI.FigureStyle.Cross;
                ol_position.Pen.Color = Color.Red;

                //番号
                FVIL.GDI.CFviGdiString ol_no = new FVIL.GDI.CFviGdiString();
                ol_no.Text = i.ToString();
                ol_no.Position = edge.Position;
                ol_no.Align = FVIL.GDI.TextAlign.Right | FVIL.GDI.TextAlign.Bottom;
                ol_no.Color = System.Drawing.Color.Red;

                //追加
                m_overlay.Figures.Add(ol_position);
                m_overlay.Figures.Add(ol_no);

                //リスト表示
                String[] listitem = { i.ToString(), edge.Position.X.ToString("0.00"), edge.Position.Y.ToString("0.00") };
                listViewResult.Items.Add(new ListViewItem(listitem));
            }
            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// x１ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void buttonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります。

            //表示倍率の設定
            cFviImageView1.Display.Magnification = 1.0;
            //スクロールバー初期化
            cFviImageView1.ScrollBarH.Value = 0;
            cFviImageView1.ScrollBarV.Value = 0;
            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void buttonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            cFviImageView1.FitImageSize();
            //スクロールバー初期化
            cFviImageView1.ScrollBarH.Value = 0;
            cFviImageView1.ScrollBarV.Value = 0;
            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void buttonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void buttonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            Value = Value - 0.5;
            if (Value <= 0)
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            cFviImageView1.Display.Magnification = Value;
            //スクロールバー初期化
            if ((cFviImageView1.Width - (cFviImageView1.Image.HorzSize * Value)) > 0)
            {
                cFviImageView1.ScrollBarH.Value = 0;
            }
            if ((cFviImageView1.Height - (cFviImageView1.Image.VertSize * Value)) > 0)
            {
                cFviImageView1.ScrollBarV.Value = 0;
            }
            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void checkBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (checkBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                cFviImageView1.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                cFviImageView1.Display.OverlayAscope.Active = false;
            }
            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void buttonSaveImage_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.SaveImageFile(ofdlg.FileName, cFviImageView1.Image);

                    //保存するファイル形式が固定の場合は、以下のように記述します。
                    //例1) bmpファイルの場合
                    //FVIL.File.CFviImageFileBmp file = new FVIL.File.CFviImageFileBmp();
                    //file.Save(ofdlg.FileName, m_image);

                    //例2) jpgファイルの場合
                    //FVIL.File.CFviImageFileJpg file = new FVIL.File.CFviImageFileJpg();
                    //file.Save(ofdlg.FileName, m_image);
                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch ((ex.ErrorCode))
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_BOX_EDGE_SAMPLE_FormClosed(object sender, FormClosedEventArgs e)
        {
            // ビデオデバイスがオープンされていた場合はクローズする
            if (m_flgVideoOpened)
            {
                m_video.Close();
                m_flgVideoOpened = false;
            }
        }

        /// <summary>
        /// リストのクリックに応じて表示を変更
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// リストのアイテムへのクリックに応じてオーバレイの表示を変更。
        /// </remarks>
        private void listViewResult_MouseUp(object sender, MouseEventArgs e)
        {
            int index = 0;

            //選択したリストのみオーバーレイを表示する、リスト以外を選択した場合全オーバーレイを表示する。
            for (int i = 0; i < listViewResult.Items.Count; i++)
            {
                ListViewItem item = listViewResult.Items[i];
                bool bEnable;

                //リスト内を選択したか判定
                if (listViewResult.SelectedItems.Count > 0)
                {
                    bEnable = (item.Checked || item.Selected);      //選択したアイテムの場合TRUEを設定し、それ以外はFALSEを設定する
                }
                else
                {
                    bEnable = true;                                 //リスト以外を選択した場合全てTRUEを設定する
                }

                //選択したアイテムを表示（未表示）にする
                m_overlay.Figures[index].Enable = bEnable;
                m_overlay.Figures[index + 1].Enable = bEnable;
                index = index + 2;
            }
            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// エッジ検出閾値のエラー処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エッジ検出閾値に0が入力された際に値を-1に変更する
        /// </remarks>
        private void numericUpDownThreshold_ValueChanged(object sender, EventArgs e)
        {
            if ((int)numericUpDownThreshold.Value == 0)
            {
                numericUpDownThreshold.Value = -1;
                //エラー処理
                MessageBox.Show("範囲:-1,1～255", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 検出線表示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 検出線表示関数
        /// </remarks>
        private void fnLinesDisplay()
        {
            //検出線表示初期化
            m_overlay_box.Figures.Clear();
            m_lines.Clear();

            //検出線
            double width = m_box.Width / ((int)numericUpDownNumber.Value - 1);
            double height = m_box.Height / ((int)numericUpDownNumber.Value - 1);

            for (int i = 0; i < numericUpDownNumber.Value; i++)
            {
                FVIL.GDI.CFviGdiLineSegment ls = new FVIL.GDI.CFviGdiLineSegment();
                ls.Pen.Color = Color.Yellow;
                switch (comboBoxScanDirection.SelectedIndex)
                {
                    //水平方向の線分
                    case 0:
                    case 1:
                        ls.St = new FVIL.Data.CFviPoint(m_box.St.X + 1, m_box.St.Y + height * i);
                        ls.Ed = new FVIL.Data.CFviPoint(m_box.Ed.X + 1, m_box.St.Y + height * i);
                        break;
                    //垂直方向の線分
                    case 2:
                    case 3:
                        ls.St = new FVIL.Data.CFviPoint(m_box.St.X + width * i, m_box.St.Y + 1);
                        ls.Ed = new FVIL.Data.CFviPoint(m_box.St.X + width * i, m_box.Ed.Y + 1);
                        break;
                }
                m_lines.Add(ls);
            }
            //追加
            for (int i = 0; i < m_lines.Count; i++)
            {
                m_overlay_box.Figures.Add(m_lines[i]);
            }

            //検出線方向表示
            switch (comboBoxScanDirection.SelectedIndex)
            {
                //右方向
                case 0:
                    m_point1.X = m_box.St.X + ((m_box.Ed.X - m_box.St.X) / 2);
                    m_point1.Y = m_box.St.Y;
                    m_point1.Angle = new FVIL.Data.CFviAngle(0.00);
                    m_point2.X = m_box.St.X + ((m_box.Ed.X - m_box.St.X) / 2);
                    m_point2.Y = m_box.Ed.Y;
                    m_point2.Angle = new FVIL.Data.CFviAngle(0.00);
                    break;
                //左方向
                case 1:
                    m_point1.X = m_box.St.X + ((m_box.Ed.X - m_box.St.X) / 2);
                    m_point1.Y = m_box.St.Y;
                    m_point1.Angle = new FVIL.Data.CFviAngle(180.00);
                    m_point2.X = m_box.St.X + ((m_box.Ed.X - m_box.St.X) / 2);
                    m_point2.Y = m_box.Ed.Y;
                    m_point2.Angle = new FVIL.Data.CFviAngle(180.00);
                    break;
                //下方向
                case 2:
                    m_point1.X = m_box.St.X;
                    m_point1.Y = m_box.St.Y + ((m_box.Ed.Y - m_box.St.Y) / 2);
                    m_point1.Angle = new FVIL.Data.CFviAngle(90.00);
                    m_point2.X = m_box.Ed.X;
                    m_point2.Y = m_box.St.Y + ((m_box.Ed.Y - m_box.St.Y) / 2);
                    m_point2.Angle = new FVIL.Data.CFviAngle(90.00);
                    break;
                //上方向
                case 3:
                    m_point1.X = m_box.St.X;
                    m_point1.Y = m_box.St.Y + ((m_box.Ed.Y - m_box.St.Y) / 2);
                    m_point1.Angle = new FVIL.Data.CFviAngle(270.00);
                    m_point2.X = m_box.Ed.X;
                    m_point2.Y = m_box.St.Y + ((m_box.Ed.Y - m_box.St.Y) / 2);
                    m_point2.Angle = new FVIL.Data.CFviAngle(270.00);
                    break;
            }
            m_overlay_box.Figures.Add(m_point1);
            m_overlay_box.Figures.Add(m_point2);

            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// マウスカーソルが動いた時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// マウスカーソルが動いた時に検出線を再描画
        /// </remarks>
        protected virtual void OnMouseMove(System.Object sender, System.EventArgs e)
        {
            if (m_FigureOverlay.MouseInfo.bGrip == true)
            {
                //表示初期化
                m_overlay.Figures.Clear();
                listViewResult.Items.Clear();
                //検出線表示
                fnLinesDisplay();
            }
        }

        /// <summary>
        /// 検出線数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 検出線数の表示
        /// </remarks>
        private void numericUpDownNumber_ValueChanged(object sender, EventArgs e)
        {
            //表示初期化
            m_overlay.Figures.Clear();
            listViewResult.Items.Clear();
            //検出線表示
            fnLinesDisplay();
        }

        /// <summary>
        /// 検出線方向
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 検出線方向の表示
        /// </remarks>
        private void comboBoxScanDirection_SelectionChangeCommitted(object sender, EventArgs e)
        {
            //表示初期化
            m_overlay.Figures.Clear();
            listViewResult.Items.Clear();
            //検出線表示
            fnLinesDisplay();
        }

    }
}